#define CORE_NAME "arp"

#include <hermit.h>
#include <io.h>
#include <string.h>

#include <net/eth_util.h>
#include <net/eth.h>
#include <net/arp.h>
#include <arch/byteorder.h>

#define LEN_ARP_Cache 16
static arp_cache_table arp_cache[LEN_ARP_Cache];
static int arp_cache_idx = 0;

void arp_cache_clear(void)
{
	arp_cache_idx = 0;
	memset(&arp_cache, 0, sizeof(arp_cache_table) * LEN_ARP_Cache);
}

static int arp_cache_add(const unsigned char *target_mac,
			 const unsigned char *target_ipaddr)
{
	int i;

	for (i=0; i<LEN_ARP_Cache; i++)
		if (memcmp(arp_cache[i].ipaddr, target_ipaddr, 4) == 0)
			break;

	if (i == LEN_ARP_Cache)
		i = (arp_cache_idx++) & 0xf;

	memcpy(arp_cache[i].mac, target_mac, 6);
	memcpy(arp_cache[i].ipaddr, target_ipaddr, 4);

	return 0;
}

static int arp_cache_search(unsigned char *target_mac,
			    const unsigned char *target_ipaddr)
{
	int i;

	for (i=0; i<LEN_ARP_Cache; i++) {
		if (memcmp(arp_cache[i].ipaddr, target_ipaddr, 4) == 0) {
			memcpy(target_mac, arp_cache[i].mac, 6);
			return 0;
		}
	}
	return -1;
}

static int arp_request(const unsigned char *target_ipaddr)
{
	eth_frame ethfr;
	arp_frame arpfr;

	memset(&ethfr, 0, sizeof(eth_frame));
	memset(&arpfr, 0, sizeof(arp_frame));

	arpfr.htype = htons(ARP_HTYPE_ETH);
	arpfr.ptype = htons(ARP_PTYPE_IP);
	arpfr.hlen = ARP_HLEN_ETH;
	arpfr.plen = ARP_PLEN_IPV4;
	arpfr.opecode = htons(ARP_OP_REQ);
	memcpy(arpfr.host_mac, eth_mac, 6);
	memcpy(arpfr.host_ipaddr, eth_ipaddr, 4);
	memset(arpfr.target_mac, 0, 6);
	memcpy(arpfr.target_ipaddr, target_ipaddr, 4);

	memcpy(ethfr.dmac, broadcast_mac, 6);
	memcpy(ethfr.smac, eth_mac, 6);
	ethfr.protocol = htons(ETH_PROTOCOL_ARP);

	return eth_send(&ethfr, &arpfr, sizeof(arp_frame));
}

int arp_search(unsigned char *target_mac, const unsigned char *target_ipaddr)
{
	int i;
	int ret;

	ret = arp_cache_search(target_mac, target_ipaddr);
	if (ret == 0)
		return 0;

	for (i=0; i<5; i++) {
		ret = arp_request(target_ipaddr);
		ret = eth_proc(ARP_WAIT, 0, 0);
		if (ret == -1)
			continue;
		ret = arp_cache_search(target_mac, target_ipaddr);
		if (ret == 0)
			return 0;
	}
	return -1;
}

int arp_proc(const unsigned char *pbuf, const int pbuflen)
{
	arp_frame *recv_arpfr = (arp_frame *)pbuf;
	arp_frame send_arpfr;
	eth_frame send_ethfr;

	if (pbuflen < sizeof(arp_frame))
		return -1;

	dev_dbg("htype     : %p\n", ntohs(recv_arpfr->htype));
	dev_dbg("ptype     : %p\n", ntohs(recv_arpfr->ptype));
	dev_dbg("hlen      : %p\n", recv_arpfr->hlen);
	dev_dbg("plen      : %p\n", recv_arpfr->plen);
	dev_dbg("opcode    : %p\n", ntohs(recv_arpfr->opecode));
	dev_dbg("hostmac   : %b:%b:%b:%b:%b:%b\n",
		recv_arpfr->host_mac[0], recv_arpfr->host_mac[1],
		recv_arpfr->host_mac[2], recv_arpfr->host_mac[3],
		recv_arpfr->host_mac[4], recv_arpfr->host_mac[5]);
	dev_dbg("hostip    : %d.%d.%d.%d\n",
		recv_arpfr->host_ipaddr[0], recv_arpfr->host_ipaddr[1],
		recv_arpfr->host_ipaddr[2], recv_arpfr->host_ipaddr[3]);
	dev_dbg("targetmac : %b:%b:%b:%b:%b:%b\n",
		recv_arpfr->target_mac[0], recv_arpfr->target_mac[1],
		recv_arpfr->target_mac[2], recv_arpfr->target_mac[3],
		recv_arpfr->target_mac[4], recv_arpfr->target_mac[5]);
	dev_dbg("targetip  : %d.%d.%d.%d\n",
		recv_arpfr->target_ipaddr[0], recv_arpfr->target_ipaddr[1],
		recv_arpfr->target_ipaddr[2], recv_arpfr->target_ipaddr[3]);

	switch (ntohs(recv_arpfr->opecode)) {
	case ARP_OP_REQ:
		if (memcmp(recv_arpfr->target_ipaddr, eth_ipaddr, 4) == 0) {
			memset(&send_arpfr, 0 , sizeof(arp_frame));
			send_arpfr.htype = htons(ARP_HTYPE_ETH);
			send_arpfr.ptype = htons(ARP_PTYPE_IP);
			send_arpfr.hlen = ARP_HLEN_ETH;
			send_arpfr.plen = ARP_PLEN_IPV4;
			send_arpfr.opecode = htons(ARP_OP_REPLY);
			memcpy(send_arpfr.host_mac, eth_mac, 6);
			memcpy(send_arpfr.host_ipaddr, eth_ipaddr, 4);
			memcpy(send_arpfr.target_mac, recv_arpfr->host_mac, 6);
			memcpy(send_arpfr.target_ipaddr,
			       recv_arpfr->host_ipaddr, 4);

			memcpy(send_ethfr.dmac, recv_arpfr->host_mac, 6);
			memcpy(send_ethfr.smac, eth_mac, 6);
			send_ethfr.protocol = htons(ETH_PROTOCOL_ARP);

			eth_send(&send_ethfr, &send_arpfr, sizeof(arp_frame));
		}
		return 0;

	case ARP_OP_REPLY:
		if (memcmp(recv_arpfr->target_ipaddr, eth_ipaddr, 4) == 0)
			arp_cache_add(recv_arpfr->host_mac,
				      recv_arpfr->host_ipaddr);
		return 0;

	case ARP_OP_RARPREQ:
	case ARP_OP_RARPREPLY:
	default:
		return -1;
	}
}
